@@ -1,5 +1,5 @@
 
-__version__ = "0.2.12"
+__version__ = "0.2.16"
 __common_alias__ = "nsbatwm"
 
 from napari.types import ImageData, LabelsData
@@ -58,10 +58,17 @@ def napari_experimental_provide_function():
         sum_images,
         multiply_images,
         divide_images,
-        invert_image
+        invert_image,
+        skeletonize,
+        Manually_merge_labels,
+        Manually_split_labels
     ]
 
 
+
+
+
+
 def _sobel_3d(image):
     kernel = np.asarray([
         [
@@ -199,13 +206,13 @@ def percentile_filter(image: napari.types.ImageData, percentile : float = 50, ra
 @register_function(menu="Filtering / background removal > White top-hat (scipy, nsbatwm)")
 @time_slicer
 def white_tophat(image: napari.types.ImageData, radius: float = 2, viewer: napari.Viewer = None) -> napari.types.ImageData:
-    return scipy.ndimage.white_tophat(image.astype(float), size=radius * 2 + 1)
+    return scipy.ndimage.white_tophat(image.astype(float), size=int(radius * 2 + 1))
 
 
 @register_function(menu="Filtering / background removal > Black top-hat (scipy, nsbatwm)")
 @time_slicer
 def black_tophat(image: napari.types.ImageData, radius: float = 2, viewer: napari.Viewer = None) -> napari.types.ImageData:
-    return scipy.ndimage.black_tophat(image.astype(float), size=radius * 2 + 1)
+    return scipy.ndimage.black_tophat(image.astype(float), size=int(radius * 2 + 1))
 
 
 @register_function(menu="Filtering / background removal > Minimum (scipy, nsbatwm)")
@@ -223,7 +230,7 @@ def maximum_filter(image: napari.types.ImageData, radius: float = 2, viewer: nap
 @register_function(menu="Filtering / background removal > Morphological Gradient (scipy, nsbatwm)")
 @time_slicer
 def morphological_gradient(image: napari.types.ImageData, radius: float = 2, viewer: napari.Viewer = None) -> napari.types.ImageData:
-    return scipy.ndimage.morphological_gradient(image.astype(float), size=radius * 2 + 1)
+    return scipy.ndimage.morphological_gradient(image.astype(float), size=int(radius * 2 + 1))
 
 
 @register_function(menu="Filtering / background removal > Rolling ball (scikit-image, nsbatwm)")
@@ -426,3 +433,70 @@ def invert_image(image: ImageData,
     from skimage import util
     return util.invert(image)
 
+
+@register_function(menu="Segmentation post-processing > Skeletonize (scikit-image, nsbatwm)")
+@time_slicer
+def skeletonize(image: ImageData,
+                                 viewer: napari.Viewer = None) -> ImageData:
+    from skimage import morphology
+    return morphology.skeletonize(image)
+
+
+@register_function(menu="Utilities > Manually merge labels (nsbatwm)")
+def Manually_merge_labels(labels_layer: napari.layers.Labels, points_layer: napari.layers.Points, viewer : napari.Viewer):
+    if points_layer is None:
+        points_layer = viewer.add_points([])
+        points_layer.mode = 'ADD'
+        return
+    labels = labels_layer.data
+    points = points_layer.data
+
+    label_ids = [labels.item(tuple([int(j) for j in i])) for i in points]
+
+    # replace labels with minimum of the selected labels
+    new_label_id = min(label_ids)
+    for l in label_ids:
+        if l != new_label_id:
+            labels[labels == l] = new_label_id
+
+    labels_layer.data = labels
+    points_layer.data = []
+
+@register_function(menu="Utilities > Manually split labels (nsbatwm)")
+def Manually_split_labels(labels_layer: napari.layers.Labels, points_layer: napari.layers.Points, viewer: napari.Viewer):
+    if points_layer is None:
+        points_layer = viewer.add_points([])
+        points_layer.mode = 'ADD'
+        return
+
+    labels = labels_layer.data
+    points = points_layer.data
+
+    label_ids = [labels.item(tuple([int(j) for j in i])) for i in points]
+
+    # make a binary image first
+    binary = np.zeros(labels.shape, dtype=bool)
+    new_label_id = min(label_ids)
+    for l in label_ids:
+        binary[labels == l] = True
+
+    # origin: https://scikit-image.org/docs/dev/auto_examples/segmentation/plot_watershed.html
+    from scipy import ndimage as ndi
+    from skimage.segmentation import watershed
+    #from skimage.feature import peak_local_max
+
+    #distance = ndi.distance_transform_edt(binary)
+    #coords = peak_local_max(distance, footprint=np.ones((3, 3)), labels=binary)
+    mask = np.zeros(labels.shape, dtype=bool)
+    for i in points:
+        #mask[tuple(points)] = True
+        mask[tuple([int(j) for j in i])] = True
+
+    markers, _ = ndi.label(mask)
+    new_labels = watershed(binary, markers, mask=binary)
+    labels[binary] = new_labels[binary] + labels.max()
+
+    labels_layer.data = labels
+    points_layer.data = []
+
+
